@@ -559,7 +559,7 @@ real_directory_notify_files_removed (NautilusDirectory *real_directory)
 		nautilus_directory_notify_files_removed_by_uri (files);
 	}
 
-	eel_g_list_free_deep (files);
+	g_list_free_full (files, g_free);
 }
 
 static void
