@@ -212,16 +212,26 @@ static GtkTreeModel *nautilus_shortcuts_model_filter_new (NautilusPlacesSidebar
 G_DEFINE_TYPE (NautilusPlacesSidebar, nautilus_places_sidebar, GTK_TYPE_SCROLLED_WINDOW);
 
 static GdkPixbuf *
-get_eject_icon (gboolean highlighted)
+get_eject_icon (NautilusPlacesSidebar *sidebar,
+		gboolean highlighted)
 {
 	GdkPixbuf *eject;
-	NautilusIconInfo *eject_icon_info;
+	GtkIconInfo *icon_info;
+	GIcon *icon;
 	int icon_size;
+	GtkIconTheme *icon_theme;
+	GtkStyleContext *style;
 
-	icon_size = nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU);
+	icon_theme = gtk_icon_theme_get_default ();
+	icon_size = nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_SMALL_TOOLBAR);
+	icon = g_themed_icon_new_with_default_fallbacks ("media-eject-symbolic");
+	icon_info = gtk_icon_theme_lookup_by_gicon (icon_theme, icon, icon_size, 0);
 
-	eject_icon_info = nautilus_icon_info_lookup_from_name ("media-eject", icon_size);
-	eject = nautilus_icon_info_get_pixbuf_at_size (eject_icon_info, icon_size);
+	style = gtk_widget_get_style_context (GTK_WIDGET (sidebar));
+	eject = gtk_icon_info_load_symbolic_for_context (icon_info,
+							 style,
+							 NULL,
+							 NULL);
 
 	if (highlighted) {
 		GdkPixbuf *high;
@@ -230,7 +240,8 @@ get_eject_icon (gboolean highlighted)
 		eject = high;
 	}
 
-	g_object_unref (eject_icon_info);
+	g_object_unref (icon);
+	gtk_icon_info_free (icon_info);
 
 	return eject;
 }
@@ -354,7 +365,7 @@ add_place (NautilusPlacesSidebar *sidebar,
 	}
 
 	if (show_eject_button) {
-		eject = get_eject_icon (FALSE);
+		eject = get_eject_icon (sidebar, FALSE);
 	} else {
 		eject = NULL;
 	}
@@ -648,20 +659,24 @@ update_places (NautilusPlacesSidebar *sidebar)
 		g_free (mount_uri);
 	}
 
-	/* desktop */
-	mount_uri = g_filename_to_uri (desktop_path, NULL, NULL);
-	icon = g_themed_icon_new (NAUTILUS_ICON_DESKTOP);
-	last_iter = add_place (sidebar, PLACES_BUILT_IN,
-			       SECTION_COMPUTER,
-			       _("Desktop"), icon,
-			       mount_uri, NULL, NULL, NULL, 0,
-			       _("Open the contents of your desktop in a folder"));
-	g_object_unref (icon);
-	compare_for_selection (sidebar,
-			       location, mount_uri, last_uri,
-			       &last_iter, &select_path);
-	g_free (mount_uri);
-	g_free (desktop_path);
+
+	if (g_settings_get_boolean (gnome_background_preferences, NAUTILUS_PREFERENCES_SHOW_DESKTOP) &&
+	    !g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR)) {
+		/* desktop */
+		mount_uri = g_filename_to_uri (desktop_path, NULL, NULL);
+		icon = g_themed_icon_new (NAUTILUS_ICON_DESKTOP);
+		last_iter = add_place (sidebar, PLACES_BUILT_IN,
+				       SECTION_COMPUTER,
+				       _("Desktop"), icon,
+				       mount_uri, NULL, NULL, NULL, 0,
+				       _("Open the contents of your desktop in a folder"));
+		g_object_unref (icon);
+		compare_for_selection (sidebar,
+				       location, mount_uri, last_uri,
+				       &last_iter, &select_path);
+		g_free (mount_uri);
+		g_free (desktop_path);
+	}
 
 	/* file system root */
  	mount_uri = "file:///"; /* No need to strdup */
@@ -688,7 +703,11 @@ update_places (NautilusPlacesSidebar *sidebar)
 
 		path = g_get_user_special_dir (index);
 
-		if (!path) {
+		/* xdg resets special dirs to the home directory in case
+		 * it's not finiding what it expects. We don't want the home
+		 * to be added multiple times in that weird configuration.
+		 */
+		if (!path || g_strcmp0 (path, g_get_home_dir ()) == 0) {
 			continue;
 		}
 
@@ -990,10 +1009,10 @@ clicked_eject_button (NautilusPlacesSidebar *sidebar,
 }
 
 static void
-desktop_location_changed_callback (gpointer user_data)
+desktop_setting_changed_callback (gpointer user_data)
 {
 	NautilusPlacesSidebar *sidebar;
-	
+
 	sidebar = NAUTILUS_PLACES_SIDEBAR (user_data);
 
 	update_places (sidebar);
@@ -1641,8 +1660,6 @@ bookmarks_check_popup_sensitivity (NautilusPlacesSidebar *sidebar)
 				    -1);
 	}
 
-	gtk_widget_show (sidebar->popup_menu_open_in_new_tab_item);
-
 	gtk_widget_set_sensitive (sidebar->popup_menu_remove_item, (type == PLACES_BOOKMARK));
 	gtk_widget_set_sensitive (sidebar->popup_menu_rename_item, (type == PLACES_BOOKMARK));
 	gtk_widget_set_sensitive (sidebar->popup_menu_empty_trash_item, !nautilus_trash_monitor_is_empty ());
@@ -1729,7 +1746,6 @@ volume_mounted_cb (GVolume *volume,
 		if (sidebar->go_to_after_mount_slot != NULL) {
 			if ((sidebar->go_to_after_mount_flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) == 0) {
 				nautilus_window_slot_open_location (sidebar->go_to_after_mount_slot, location,
-								    NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
 								    sidebar->go_to_after_mount_flags, NULL);
 			} else {
 				NautilusApplication *app;
@@ -1737,9 +1753,9 @@ volume_mounted_cb (GVolume *volume,
 
 				cur = NAUTILUS_WINDOW (sidebar->window);
 				app = nautilus_application_dup_singleton ();
-				new = nautilus_application_create_navigation_window (app,
-										     NULL,
-										     gtk_window_get_screen (GTK_WINDOW (cur)));
+				new = nautilus_application_create_window (app,
+									  NULL,
+									  gtk_window_get_screen (GTK_WINDOW (cur)));
 				nautilus_window_go_to (new, location);
 
 				g_object_unref (app);
@@ -1807,7 +1823,6 @@ open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 		if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) == 0) {
 			slot = nautilus_window_get_active_slot (sidebar->window);
 			nautilus_window_slot_open_location (slot, location,
-							    NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
 							    flags, NULL);
 		} else {
 			NautilusWindow *cur, *new;
@@ -1815,9 +1830,9 @@ open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 			
 			cur = NAUTILUS_WINDOW (sidebar->window);
 			app = nautilus_application_dup_singleton ();
-			new = nautilus_application_create_navigation_window (app,
-									     NULL,
-									     gtk_window_get_screen (GTK_WINDOW (cur)));
+			new = nautilus_application_create_window (app,
+								  NULL,
+								  gtk_window_get_screen (GTK_WINDOW (cur)));
 			nautilus_window_go_to (new, location);
 
 			g_object_unref (app);
@@ -1910,8 +1925,17 @@ rename_selected_bookmark (NautilusPlacesSidebar *sidebar)
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *cell;
 	GList *renderers;
+	PlaceType type;
 	
 	if (get_selected_iter (sidebar, &iter)) {
+		gtk_tree_model_get (GTK_TREE_MODEL (sidebar->filter_model), &iter,
+				    PLACES_SIDEBAR_COLUMN_ROW_TYPE, &type,
+				    -1);
+
+		if (type != PLACES_BOOKMARK) {
+			return;
+		}
+
 		path = gtk_tree_model_get_path (GTK_TREE_MODEL (sidebar->filter_model), &iter);
 		column = gtk_tree_view_get_column (GTK_TREE_VIEW (sidebar->tree_view), 0);
 		renderers = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (column));
@@ -2400,6 +2424,58 @@ empty_trash_cb (GtkMenuItem           *item,
 	nautilus_file_operations_empty_trash (GTK_WIDGET (sidebar->window));
 }
 
+static gboolean
+find_prev_or_next_row (NautilusPlacesSidebar *sidebar,
+		       GtkTreeIter *iter,
+		       gboolean go_up)
+{
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	int place_type;
+	gboolean res;
+
+	selection = gtk_tree_view_get_selection (sidebar->tree_view);
+	res = gtk_tree_selection_get_selected (selection, &model, iter);
+
+	if (!res) {
+		goto out;
+	}
+
+	if (go_up) {
+		res = gtk_tree_model_iter_previous (model, iter);
+	} else {
+		res = gtk_tree_model_iter_next (model, iter);
+	}
+
+	if (res) {
+		gtk_tree_model_get (model, iter,
+				    PLACES_SIDEBAR_COLUMN_ROW_TYPE, &place_type,
+				    -1);
+		if (place_type == PLACES_HEADING) {
+			if (go_up) {
+				res = gtk_tree_model_iter_previous (model, iter);
+			} else {
+				res = gtk_tree_model_iter_next (model, iter);
+			}
+		}
+	}
+
+ out:
+	return res;
+}
+
+static gboolean
+find_prev_row (NautilusPlacesSidebar *sidebar, GtkTreeIter *iter)
+{
+	return find_prev_or_next_row (sidebar, iter, TRUE);
+}
+
+static gboolean
+find_next_row (NautilusPlacesSidebar *sidebar, GtkTreeIter *iter)
+{
+	return find_prev_or_next_row (sidebar, iter, FALSE);
+}
+
 /* Handler for GtkWidget::key-press-event on the shortcuts list */
 static gboolean
 bookmarks_key_press_event_cb (GtkWidget             *widget,
@@ -2407,6 +2483,7 @@ bookmarks_key_press_event_cb (GtkWidget             *widget,
 			      NautilusPlacesSidebar *sidebar)
 {
   guint modifiers;
+  GtkTreeIter iter;
 
   modifiers = gtk_accelerator_get_default_mod_mask ();
 
@@ -2439,6 +2516,22 @@ bookmarks_key_press_event_cb (GtkWidget             *widget,
       return eject_or_unmount_selection (sidebar);
   }
 
+  if (event->keyval == GDK_KEY_Up) {
+      if (find_prev_row (sidebar, &iter)) {
+	      gtk_tree_selection_select_iter (gtk_tree_view_get_selection (sidebar->tree_view),
+					      &iter);
+      }
+      return TRUE;
+  }
+
+  if (event->keyval == GDK_KEY_Down) {
+      if (find_next_row (sidebar, &iter)) {
+	      gtk_tree_selection_select_iter (gtk_tree_view_get_selection (sidebar->tree_view),
+					      &iter);
+      }
+      return TRUE;
+  }
+
   if ((event->keyval == GDK_KEY_Delete
       || event->keyval == GDK_KEY_KP_Delete)
       && (event->state & modifiers) == 0) {
@@ -2460,11 +2553,15 @@ static void
 bookmarks_build_popup_menu (NautilusPlacesSidebar *sidebar)
 {
 	GtkWidget *item;
+	gboolean use_browser;
 	
 	if (sidebar->popup_menu) {
 		return;
 	}
-	
+
+	use_browser = g_settings_get_boolean (nautilus_preferences,
+					      NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER);
+
 	sidebar->popup_menu = gtk_menu_new ();
 	gtk_menu_attach_to_widget (GTK_MENU (sidebar->popup_menu),
 			           GTK_WIDGET (sidebar),
@@ -2482,15 +2579,21 @@ bookmarks_build_popup_menu (NautilusPlacesSidebar *sidebar)
 	sidebar->popup_menu_open_in_new_tab_item = item;
 	g_signal_connect (item, "activate",
 			  G_CALLBACK (open_shortcut_in_new_tab_cb), sidebar);
-	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (sidebar->popup_menu), item);
 
+	if (use_browser) {
+		gtk_widget_show (item);
+	}
+
 	item = gtk_menu_item_new_with_mnemonic (_("Open in New _Window"));
 	g_signal_connect (item, "activate",
 			  G_CALLBACK (open_shortcut_in_new_window_cb), sidebar);
-	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (sidebar->popup_menu), item);
 
+	if (use_browser) {
+		gtk_widget_show (item);
+	}
+
 	eel_gtk_menu_append_separator (GTK_MENU (sidebar->popup_menu));
 
 	item = gtk_image_menu_item_new_with_label (_("Remove"));
@@ -2681,7 +2784,7 @@ update_eject_buttons (NautilusPlacesSidebar *sidebar,
 
 			gtk_list_store_set (sidebar->store,
 					    &iter,
-					    PLACES_SIDEBAR_COLUMN_EJECT_ICON, get_eject_icon (FALSE),
+					    PLACES_SIDEBAR_COLUMN_EJECT_ICON, get_eject_icon (sidebar, FALSE),
 					    -1);
 			gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (sidebar->filter_model));
 
@@ -2703,7 +2806,7 @@ update_eject_buttons (NautilusPlacesSidebar *sidebar,
 					 path);
 		gtk_list_store_set (sidebar->store,
 				    &iter,
-				    PLACES_SIDEBAR_COLUMN_EJECT_ICON, get_eject_icon (TRUE),
+				    PLACES_SIDEBAR_COLUMN_EJECT_ICON, get_eject_icon (sidebar, TRUE),
 				    -1);
 		gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (sidebar->filter_model));
 
@@ -2752,6 +2855,7 @@ bookmarks_button_press_event_cb (GtkWidget             *widget,
 		GtkTreeModel *model;
 		GtkTreePath *path;
 		GtkTreeView *tree_view;
+		NautilusWindowOpenFlags flags = 0;
 
 		tree_view = GTK_TREE_VIEW (widget);
 		g_assert (tree_view == sidebar->tree_view);
@@ -2761,10 +2865,16 @@ bookmarks_button_press_event_cb (GtkWidget             *widget,
 		gtk_tree_view_get_path_at_pos (tree_view, (int) event->x, (int) event->y, 
 					       &path, NULL, NULL, NULL);
 
-		open_selected_bookmark (sidebar, model, path,
-					event->state & GDK_CONTROL_MASK ?
-					NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW :
-					NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB);
+		if (g_settings_get_boolean (nautilus_preferences,
+					    NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
+			flags = (event->state & GDK_CONTROL_MASK) ?
+				NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW :
+				NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
+		} else {
+			flags = NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND;
+		}
+
+		open_selected_bookmark (sidebar, model, path, flags);
 
 		if (path != NULL) {
 			gtk_tree_path_free (path);
@@ -2929,7 +3039,6 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 	GtkTreeViewColumn *col;
 	GtkCellRenderer   *cell;
 	GtkTreeSelection  *selection;
-	GtkStyleContext   *style;
 
 	sidebar->volume_monitor = g_volume_monitor_get ();
 	
@@ -2938,15 +3047,15 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 					GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_hadjustment (GTK_SCROLLED_WINDOW (sidebar), NULL);
 	gtk_scrolled_window_set_vadjustment (GTK_SCROLLED_WINDOW (sidebar), NULL);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sidebar), GTK_SHADOW_IN);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sidebar), GTK_SHADOW_NONE);
+
+	gtk_style_context_set_junction_sides (gtk_widget_get_style_context (GTK_WIDGET (sidebar)),
+					      GTK_JUNCTION_RIGHT | GTK_JUNCTION_LEFT);
 
   	/* tree view */
 	tree_view = GTK_TREE_VIEW (gtk_tree_view_new ());
 	gtk_tree_view_set_headers_visible (tree_view, FALSE);
 
-	style = gtk_widget_get_style_context (GTK_WIDGET (tree_view));
-	gtk_style_context_add_class (style, "NautilusSidebar");
-
 	col = GTK_TREE_VIEW_COLUMN (gtk_tree_view_column_new ());
 
 	/* initial padding */
@@ -3117,7 +3226,11 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 							TRUE);
 
 	g_signal_connect_swapped (nautilus_preferences, "changed::" NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR,
-				  G_CALLBACK(desktop_location_changed_callback),
+				  G_CALLBACK(desktop_setting_changed_callback),
+				  sidebar);
+
+	g_signal_connect_swapped (gnome_background_preferences, "changed::" NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR,
+				  G_CALLBACK(desktop_setting_changed_callback),
 				  sidebar);
 
 	g_signal_connect_object (nautilus_trash_monitor_get (),
@@ -3154,7 +3267,11 @@ nautilus_places_sidebar_dispose (GObject *object)
 	eel_remove_weak_pointer (&(sidebar->go_to_after_mount_slot));
 
 	g_signal_handlers_disconnect_by_func (nautilus_preferences,
-					      desktop_location_changed_callback,
+					      desktop_setting_changed_callback,
+					      sidebar);
+
+	g_signal_handlers_disconnect_by_func (gnome_background_preferences,
+					      desktop_setting_changed_callback,
 					      sidebar);
 
 	G_OBJECT_CLASS (nautilus_places_sidebar_parent_class)->dispose (object);
@@ -3208,6 +3325,9 @@ nautilus_places_sidebar_set_parent_window (NautilusPlacesSidebar *sidebar,
 	g_signal_connect_object (sidebar->volume_monitor, "drive_changed",
 				 G_CALLBACK (drive_changed_callback), sidebar, 0);
 
+	g_signal_connect_swapped (nautilus_preferences, "changed::" NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER,
+				  G_CALLBACK (bookmarks_popup_menu_detach_cb), sidebar);
+
 	update_places (sidebar);
 }
 
